/*
 *  libMirage: SNDFILE fragment: Plugin exports
 *  Copyright (C) 2007 Rok Mandeljc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "fragment-sndfile.h"


static GObject *__fragment_sndfile_plugin_create_fragment () {
    g_debug("%s: start\n", __func__);
    GObject *tmp = g_object_new(MIRAGE_TYPE_FRAGMENT_SNDFILE, NULL);
    g_debug("%s: end\n", __func__);
    return tmp;
}

static gboolean __fragment_sndfile_plugin_can_handle_data (gchar *filename, gint interface) {
    /* Does file exist */
    if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
        return FALSE;
    }
    
    /* Check if we implement desired interface */
    if (interface != mirage_plugin_info.fragment_interface) {
        return FALSE;
    }
    
    /* Check supported file suffixes */
    /* FIXME: maybe calling a sf_open() and checking its return is a better idea? */
    gchar *suffixes[] = { ".wav", ".aiff", NULL };
    return mirage_helper_match_suffixes(filename, suffixes);
}

MIRAGE_PluginInfo_Fragment mirage_plugin_info = {
    .plugin_info = {
        .type = MIRAGE_PLUGIN_FRAGMENT,
        .name = "SNDFILE Fragment Plugin",
        .version = "1.0.0",
        .author = "Rok Mandeljc",
    },
    
    .fragment_id = "Fragment-SNDFILE",
    .fragment_interface = FRAGMENT_IFACE_AUDIO,

    .create_fragment = __fragment_sndfile_plugin_create_fragment,
    .can_handle_data = __fragment_sndfile_plugin_can_handle_data,
};
