/*
 *  libMirage: Plugin
 *  Copyright (C) 2006-2007 Rok Mandeljc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __MIRAGE_PLUGIN_H__
#define __MIRAGE_PLUGIN_H__


G_BEGIN_DECLS
  
enum {
    MIRAGE_PLUGIN_PARSER   = 0x01,
    MIRAGE_PLUGIN_FRAGMENT = 0x02,
};

typedef struct {
    gint type; /* Plugin type */
    
    gchar *name;    /* Plugin name */
    gchar *version; /* Plugin version */
    gchar *author;  /* Plugin author */
} MIRAGE_PluginInfo;

typedef struct {
    MIRAGE_PluginInfo plugin_info;
    
    gchar *image_id; /* Image type identifier, which will be used e.g. when forcing type of image */
    
    /* The following can be useful for building filters in Gtk applications */
    gchar *suffixes[10]; /* List of image file suffixes (e.g. ".ccd") */
    gchar *description;  /* Image file description (e.g. "CloneCD image") */
    
    GObject* (*create_disc) (void); /* Create disc object */
    gboolean (*can_handle_image) (gchar *); /* Can load image from file? */
} MIRAGE_PluginInfo_Parser;

typedef struct {
    MIRAGE_PluginInfo plugin_info;
    
    gchar *fragment_id;
    gint fragment_interface;
        
    GObject* (*create_fragment) (void); /* Create fragment object */
    gboolean (*can_handle_data) (gchar *, gint); /* Can handle data stored in file? */
} MIRAGE_PluginInfo_Fragment;


G_END_DECLS

#endif /* __MIRAGE_PLUGIN_H__ */
