/*
 *  libMirage: CUE image parser: Disc object
 *  Copyright (C) 2006-2007 Rok Mandeljc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "image-cue.h"


/* Some prototypes from flex/bison */
int yylex_init (void *scanner);
void yyset_in  (FILE *in_str, void *yyscanner);
int yylex_destroy (void *yyscanner);
int yyparse (void *scanner, MIRAGE_Session *self, GError **error);


/******************************************************************************\
 *                        Load and save implementation                        *
\******************************************************************************/
static gboolean __mirage_disc_cue_get_image_type (MIRAGE_Disc *self, gchar **image_type, GError **error) {
    *image_type = g_strdup(mirage_plugin_info.image_id);
    return TRUE;
}

static gboolean __mirage_disc_cue_load_image (MIRAGE_Disc *self, gchar **filenames, GError **error) {
    gint length = 0;
    gint i;
    
    mirage_disc_set_filenames(self, filenames, NULL);
    
    /* Each CUE/BIN is one session, so we load all given filenames */
    for (i = 0; i < g_strv_length(filenames); i++) {
        void *scanner = NULL;
        FILE *file = NULL;
        
        /* Open file */
        file = g_fopen(filenames[i], "r");
        if (!file) {
            mirage_error(MIRAGE_E_IMAGEFILE, error);
            return FALSE;
        }
        
        /* Create CUE session */
        GObject *session = g_object_new(MIRAGE_TYPE_SESSION_CUE, NULL);
        __mirage_session_cue_set_cue_filename(MIRAGE_SESSION(session), filenames[i], NULL);
        if (!mirage_disc_add_session_by_index(self, -1, &session, error)) {
            return FALSE;
        }
    
        /* Prepare scanner */
        yylex_init(&scanner);
        yyset_in(file, scanner);
        
        /* Load */
        if (yyparse(scanner, MIRAGE_SESSION(session), error)) {
            fclose(file);
            return FALSE;
        }
        
        /* Destroy scanner */
        yylex_destroy(scanner);        
        fclose(file);
        
        __mirage_session_cue_finish_last_track(MIRAGE_SESSION(session), NULL);
        
        g_object_unref(session);
    }
    
    /* Now get length and if it surpasses length of 90min CD, assume we
       have a DVD */
    mirage_disc_layout_get_length(self, &length, NULL);
    if (length > 90*60*75) {
        mirage_disc_set_medium_type(self, MIRAGE_MEDIUM_DVD, NULL);
    } else {
        mirage_disc_set_medium_type(self, MIRAGE_MEDIUM_CD, NULL);
    }
    
    return TRUE;
}

static gboolean __mirage_disc_cue_save_image (MIRAGE_Disc *self, MIRAGE_Disc *input_disc, GError **error) {
    mirage_error(MIRAGE_E_NOTIMPL, error);
    return FALSE;
}


/******************************************************************************\
 *                                Object init                                 *
\******************************************************************************/
static void __mirage_disc_cue_class_init (gpointer g_class, gpointer g_class_data) {
    MIRAGE_DiscClass *klass = MIRAGE_DISC_CLASS(g_class);

    /* Initialize MIRAGE_Disc functions */
    klass->get_image_type = __mirage_disc_cue_get_image_type;
    klass->load_image = __mirage_disc_cue_load_image;
    klass->save_image = __mirage_disc_cue_save_image;
    
    return;
}

GType mirage_disc_cue_get_type (void) {
    static GType type = 0;
    if (type == 0) {
        static const GTypeInfo info = {
            sizeof(MIRAGE_Disc_CUEClass),
            NULL,   /* base_init */
            NULL,   /* base_finalize */
            __mirage_disc_cue_class_init,   /* class_init */
            NULL,   /* class_finalize */
            NULL,   /* class_data */
            sizeof(MIRAGE_Disc_CUE),
            0,      /* n_preallocs */
            NULL    /* instance_init */
        };
        
        type = g_type_register_static(MIRAGE_TYPE_DISC, "MIRAGE_Disc_CUE", &info, 0);
    }
    
    return type;
}
