/*
 *  libMirage: ISO image parser: Plugin exports
 *  Copyright (C) 2006-2007 Rok Mandeljc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "image-iso.h"


static GObject *__image_iso_create_disc () {
    return g_object_new(MIRAGE_TYPE_DISC_ISO, NULL);
}

static gboolean __image_iso_can_handle_image (gchar *filename) {
    /* Does file exist? */
    if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
        return FALSE;
    }
    
    /* Check supported suffixes */
    if (!mirage_helper_match_suffixes(filename, mirage_plugin_info.suffixes)) {
        return FALSE;
    }
    
    /* Mode 1/Mode 2 Form 1 Images need to pass extra check */
    if (mirage_helper_has_suffix(filename, ".iso") 
        || mirage_helper_has_suffix(filename, ".img")) {
        struct stat st;
        FILE *file = NULL;
        
        /* Stat */
        if (g_stat(filename, &st) < 0) {
            return FALSE;
        }
        
        /* Since it's Mode 1/Mode 2 Form 1 track, its length should be divisible
           by 2048 */
        if (st.st_size % 2048) {
            return FALSE;
        }
    
        /* Last test; ISO9660 image has the following pattern at the beginning of 
           16th sector: 0x01, 0x43, 0x44, 0x30, 0x30, 0x31, 0x01, 0x00 */
        file = g_fopen(filename, "r");
        if (!file) {
            return FALSE;
        }
    
        guint8 buf[8] = {};
        guint8 pattern[8] = {0x01, 0x43, 0x44, 0x30, 0x30, 0x31, 0x01, 0x00};
        fseeko(file, 16*2048, SEEK_SET);
        fread(buf, 8, 1, file);
        fclose(file);
        if (memcmp(buf, pattern, 8)) {
            return FALSE;
        }
    }
    
    return TRUE;
}

MIRAGE_PluginInfo_Parser mirage_plugin_info = {
    .plugin_info = {
        .type = MIRAGE_PLUGIN_PARSER,
        .name = "ISO Image Parser",
        .version = "1.0.0",
        .author = "Rok Mandeljc",
    },
    
    .image_id = "IMAGE-ISO",
    
    .suffixes = { ".iso", ".img", ".wav", NULL },
    .description = "ISO images",
    
    .create_disc = __image_iso_create_disc,
    .can_handle_image = __image_iso_can_handle_image,
};
