/*
 *  libMirage: MDS image parser: Plugin explorts
 *  Copyright (C) 2006-2007 Rok Mandeljc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "image-mds.h"


static GObject *__image_mds_create_disc () {
    return g_object_new(MIRAGE_TYPE_DISC_MDS, NULL);
}

static gboolean __image_mds_can_handle_image (gchar *filename) {
    /* Does file exist? */
    if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
        return FALSE;
    }
    
    /* We don't check extension here: I guess we should support anything that
       passes the MEDIA DESCRIPTOR test */
    /*if (!mirage_helper_plugin_supports_suffix(filename, &plugin_info)) {
        return FALSE;
    }*/

    /* Also check that there's appropriate signature */
    FILE *file = g_fopen(filename, "r");
    if (!file) {
        return FALSE;
    }
    
    gchar sig[16] = {0};
    fread(sig, 16, 1, file);
    fclose(file);
    if (memcmp(sig, "MEDIA DESCRIPTOR", 16)) {
        return FALSE;
    }
    
    return TRUE;
}

MIRAGE_PluginInfo_Parser mirage_plugin_info = {
    .plugin_info = {
        .type = MIRAGE_PLUGIN_PARSER,
    .name = "MDS Image Parser",
        .version = "1.0.0",
        .author = "Rok Mandeljc",
    },
    
    .image_id = "IMAGE-MDS",
    
    .suffixes = { ".mds", ".xmd", NULL },
    .description = "MDS (Media descriptor) images",
    
    .create_disc = __image_mds_create_disc,
    .can_handle_image = __image_mds_can_handle_image,
};
