/*
 *  libMirage: NRG image parser: Plugin exports
 *  Copyright (C) 2006-2007 Rok Mandeljc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "image-nrg.h"


static GObject *__image_nrg_create_disc () {
    return g_object_new(MIRAGE_TYPE_DISC_NRG, NULL);
}

static gboolean __image_nrg_can_handle_image (gchar *filename) {
    gboolean can_load = FALSE;
    
    /* Does file exist? */
    if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
        return FALSE;
    }
    
    /* Check the suffixes */
    if (!mirage_helper_match_suffixes(filename, mirage_plugin_info.suffixes)) {
        return FALSE;
    }
    
    /* Also check that there's appropriate signature */
    FILE *file = g_fopen(filename, "r");
    gchar sig[4] = "";
    
    if (!file) {
        return FALSE;
    }
    
    /* New format */
    fseeko(file, -12, SEEK_END);
    fread(sig, 4, 1, file);
    if (!memcmp(sig, "NER5", 4)) {
        can_load = TRUE;
    } else {
        /* Old format */
        fseeko(file, -8, SEEK_END);
        fread(sig, 4, 1, file);
        if (!memcmp(sig, "NERO", 4)) {
            can_load = TRUE;
        }
    }

    fclose(file);
    
    return can_load;
}

MIRAGE_PluginInfo_Parser mirage_plugin_info = {
    .plugin_info = {
        .type = MIRAGE_PLUGIN_PARSER,
        .name = "NRG Image Parser",
        .version = "1.0.0",
        .author = "Rok Mandeljc",
    },
    
    .image_id = "IMAGE-NRG",
    
    .suffixes = { ".nrg", NULL },
    .description = "NRG (Nero Burning Rom) images",
    
    .create_disc = __image_nrg_create_disc,
    .can_handle_image = __image_nrg_can_handle_image,
};
